@@ -223,14 +223,12 @@ static ssize_t lirc_read(struct file * file,
 			break;
 		}
 		
-		retval=verify_area(VERIFY_WRITE,(void *) buf+n,
-				   sizeof(lirc_t));
-		if (retval)
+		if(copy_to_user((void *) buf+n,(void *) (rx_buf+rx_head),
+				sizeof(lirc_t)))
 		{
-			return retval;
+			retval = -EFAULT;
+			break;
 		}
-		copy_to_user((void *) buf+n,(void *) (rx_buf+rx_head),
-			     sizeof(lirc_t));
 		rx_head=(rx_head+1)&(RBUF_LEN-1);
 		n+=sizeof(lirc_t);
 	}
@@ -248,14 +246,10 @@ static ssize_t lirc_write(struct file * file,
 			  loff_t * pos)
 {
 	int i;
-	int retval;
 
         if(n%sizeof(lirc_t) || (n/sizeof(lirc_t)) > WBUF_LEN)
 		return(-EINVAL);
-	retval = verify_area(VERIFY_READ, buf, n);
-	if (retval)
-		return retval;
-	copy_from_user(tx_buf, buf, n);
+	if(copy_from_user(tx_buf, buf, n)) return -EFAULT;
 	i = 0;
 	n/=sizeof(lirc_t);
 	init_send();
@@ -936,6 +930,7 @@ static int init_port(void)
 		printk(KERN_ERR LIRC_DRIVER_NAME
 		       ": IRQ %d already in use.\n",
 		       irq);
+		release_region(io, 8);
 		return retval;
 	}
 
